# SPDX-License-Identifier: CC0-1.0

{ testEnv, pkgs, depot, hostPkgs, ... }:
with pkgs;

testEnv.mkTest {
  name = "driver_manager";
  meta.maintainers = with pkgs.stdenv.lib.maintainers; [ ehmry ];
  testEnv = { drivers = ./../compositions/pc-drivers.dhall; };
  testInputs = (map pkgs.genodeSources.make [
    "app/driver_manager"
    "app/rom_reporter"
    "drivers/acpi"
    "drivers/ahci"
    "drivers/framebuffer/boot"
    "drivers/framebuffer/intel"
    "drivers/framebuffer/vesa"
    "drivers/input/spec/ps2"
    "drivers/platform"
    "drivers/rtc"
    "drivers/usb"
    "server/dynamic_rom"
    "server/input_filter"
    "server/report_rom"
    "test/driver_manager"
  ]);

  testScript = ''
    catch { exec dd if=/dev/zero of=hdd_disk.raw bs=1M count=32 }
    catch { exec ${hostPkgs.e2fsprogs}/bin/mke2fs -F bin/hdd_disk.raw }
    run_genode_until {.*all expected devices present and accessible.*} 120
  '';
  testConfig = ./driver_manager.dhall;
  qemuArgs = [
    "-device ahci,id=ahci"
    "-drive id=hdd,file=hdd_disk.raw,format=raw,if=none"
    "-device ide-hd,drive=hdd,bus=ahci.1"
  ];
}
